package org.diagbot.repository;

import org.diagbot.entity.node.base.BaseNode;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;

import java.util.List;
import java.util.Map;

public interface BaseNodeRepository extends Neo4jRepository<BaseNode, Long> {
    // update Node by Name
    @Query("MATCH (n) WHERE ID(n) = {0} SET n.name = {2} RETURN n")
    BaseNode updateNodeByName(int id, String old_name, String new_name);

    // get All Node
    @Query("MATCH (n) RETURN n.name as name, LABELS(n) as label")
    List<Map<String, Object>> getAllNode();

//    Stream<BaseNode> findAllBy();
    //get high risk
    @Query("match(d:Disease) where d.name in {0} return d.name as name, d.high_risk as risk")
    List<Map<String, Object>> getHighRisk(List diagList);

    //get LIS PACS
    @Query("match (d:Disease)-[r1:推荐]->(m) where d.name in {0} return  labels(m)[0] as label,m.name as name")
    List<Map<String,Object>> getLisPacsList(List diagList);

    // get disease
    @Query("with {0} as data unwind data as row \n" +
            "match (n)-[r:属于]->(m)-[r1:组合]->(k) \n" +
            "where n.name= row \n" +
            "return m.name as condition, count(distinct r)>=m.path as jundgement, labels(m)[0] as label,k.name as standName")
    List<Map<String,Object>> getCollectionWord(List inputFilds);
    @Query("match (n)-[r:诊断依据|:近义词]->(e) where n.name in {0} return n.name as fild,collect(distinct type(r)) as typeCollect")
    List<Map<String,Object>> getCollectJYWord(List inputFilds);
    @Query("match(l)-[r:近义词]->(h) where l.name={0} return h.name as js")
    List<Map<String,Object>> getJYWord(String filds);
    @Query("with {0} as data unwind data as row \n" +
            "match (l)-[r:诊断依据]->(m) \n" +
            "where l.name= row \n" +
            "with m,{1} as data unwind data as row \n" +
            "match (n)-[r:诊断依据]->(m) \n" +
            "where n.name= row \n" +
            "return m.name as condition, count(distinct r)>=m.path as jundgement, labels(m)[0] as label")
    List<Map<String,Object>> getCondition(List newList,List inputList);
    @Query("with {0} as data unwind data as row \n" +
            "match (n)-[r:确诊|:拟诊]->(m)\n" +
            "where n.name=row \n" +
            "with distinct m,r \n" +
            "return m.name as name, labels(m)[0] as label,type(r) as relationType")
    List<Map<String,Object>> getDisease(List fildList);
    //get treat
    @Query("match (d:Disease{name:{0}})-[r0:推荐]->(m:Drugs)-[:包含]->(n:Medicine),(d)-[r1:推荐]->(n) \n" +
            "where r0.p<6 \n" +
            "return m.name as drugs, r0.p as sort, n.name as medication, r1.rate as rate order by sort ,rate desc")
    List<Map<String,Object>> getTreatMedicaton(String dis);
    @Query("match (n:Disease{name:{0}})-[r0:推荐]->(m:Drugs)-[:包含]->(k:Drugs) return k.name as short,m.name as large")
    List<Map<String,Object>> getShortLarge(String dis);
    @Query("match (n:Disease{name:{0}})-[:推荐]->(d)<-[r0:慎用]-(m) \n" +
            "where m.name in {1} and labels(d)[0] in ['Drugs','Medicine'] return d.name as name,type(r0) as type \n" +
            "union \n" +
            "match (n:Disease{name:{0}})-[:推荐]->(d)<-[r0:忌用]-(m) where m.name in {1} and labels(d)[0] in ['Drugs','Medicine'] \n" +
            "return d.name as name,type(r0) as type")
    List<Map<String,Object>> getShenJiDrugsMedication(String dis,List filds);

}
