package org.diagbot.service.impl;

import com.alibaba.fastjson.JSONObject;
import org.diagbot.entity.node.LIS;
import org.diagbot.entity.node.Symptom;
import org.diagbot.mapper.KnowledgeMapper;
import org.diagbot.service.KnowledgeService;
import org.diagbot.vo.*;
import org.diagbot.entity.node.Disease;
import org.diagbot.entity.node.base.BaseNode;
import org.apache.commons.collections4.IteratorUtils;
import org.diagbot.repository.*;
import org.diagbot.vo.domain.*;
import org.neo4j.driver.v1.types.Node;
import org.neo4j.driver.v1.types.Path;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.*;


@Service
public class KnowledgeServiceImpl implements KnowledgeService {
    Logger logger = LoggerFactory.getLogger(KnowledgeServiceImpl.class);
    @Autowired
    private SymptomRepository symptomRepository;
    @Autowired
    private DiseaseRepository diseaseRepository;
    @Autowired
    private LISRepository lisRepository;
    @Autowired
    private BaseNodeRepository baseNodeRepository;
    @Autowired
    private RelationRepository relationRepository;
    @Autowired
    private ScaleRepository scaleRepository;
    @Autowired
    private KnowledgeMapper knowledgeMapper;

    private List<BaseNode> baseNodes;

    private Pageable pageable;
    /**
     * 处理症状节点相关的申请
     */
    @Override
    public Symptom addSymptom(Symptom symptom) {
        return symptomRepository.save(symptom);
    }

    @Override
    public Symptom findOneSymptom(long id) {
        return symptomRepository.findById(id).get();
    }

    @Override
    public List<Symptom> findSymptoms(String name) { return symptomRepository.findByNameContaining(name); }

    @Override
    public void deleteOneSymptom(long id) {
        symptomRepository.deleteById(id);
    }

    @Override
    public List<Symptom> selectAllSymptom(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(symptomRepository.findAll().iterator());
    }


    /**
     * 处理疾病节点相关的申请
     */
    @Override
    public Disease addDisease(Disease disease) {
        return diseaseRepository.save(disease);
    }

    @Override
    public Disease findOneDisease(long id) {
        return diseaseRepository.findById(id).get();
    }

    @Override
    public List<Disease> findDisease(String name) {
        return diseaseRepository.findByNameContaining(name);
    }

    @Override
    public void deleteOneDisease(long id) {diseaseRepository.deleteById(id); }

    @Override
    public List<Disease> selectAllDisease(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(diseaseRepository.findAll(pageable).iterator());
    }


    /**
     * 处理化验节点相关的申请
     */
    @Override
    public LIS addLIS(LIS lis) {
        return lisRepository.save(lis);
    }

    @Override
    public LIS findOneLIS(long id) {
        return lisRepository.findById(id).get();
    }

    @Override
    public List<LIS> findLIS(String name) { return lisRepository.findByNameContaining(name); }

    @Override
    public void deleteOneLIS(long id) {lisRepository.deleteById(id); }

    @Override
    public List<LIS> selectAllLIS(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(lisRepository.findAll().iterator());
    }


    @Override
    public List<Map<String, Object>> getRelationByNodes(RelationVo relationVo) {
        String s_name = relationVo.getStart_name();
        String s_type = relationVo.getStart_type();
        String e_name = relationVo.getEnd_name();
        String e_type = relationVo.getEnd_type();

        List<Map<String, Object>> list = extractPath(relationRepository.getRelationByNodes(s_name, s_type, e_name, e_type));
        return list;
    }

    @Override
    public List<Map<String, Object>> getOneLevelRelationByEndNode(RelationVo relationVo) {
        String e_name = relationVo.getEnd_name();
        String e_type = relationVo.getEnd_type();

        List<Map<String, Object>> list = extractPath(relationRepository.getOneLevelRelationByEndNode(e_name, e_type));
        return list;
    }

    @Override
    public Iterable<Map<String, Object>> getNodeByRelation(RelationVo relationVo) {
        String s_name = relationVo.getStart_name();
        String s_type = relationVo.getStart_type();
        String rel = relationVo.getRel_name();

        return relationRepository.getNodeByRelation(s_name, s_type, rel);
    }

    @Override
    public BaseNode updateNodeByName(AmendRelationVo amendRelationVo) {
        int id = amendRelationVo.getId();
        String old_name = amendRelationVo.getOld_start_name();
        String new_name = amendRelationVo.getNew_start_name();

        return baseNodeRepository.updateNodeByName(id, old_name, new_name);
    }

    @Override
    public List<Map<String, Object>> updateRelation(AmendRelationVo amendRelationVo) {
        List<Map<String, Object>> path = extractPath(knowledgeMapper.updateRelation(amendRelationVo));
        return path;
    }

    @Override
    public List<Map<String, Object>> getAllNode() {
        List<Map<String, Object>> ndlist = baseNodeRepository.getAllNode();
        return ndlist;
    }

    @Override
    public Map<String, Object> getHighRiskDisease(SearchData searchData) {
        Map<String, Object> map = new HashMap<>();
        String diag = searchData.getDiag();
        if(diag != null && diag != ""){
            String[] splitsDiag = searchData.getDiag().split(",|，|、");
            List<String> diseaseNameList = Arrays.asList(splitsDiag);
            List<Map<String, Object>> list = baseNodeRepository.getHighRisk(diseaseNameList);
            for (Map hrmap:list) {
                String name = hrmap.get("name").toString();
                String risk = hrmap.get("risk").toString();
                map.put(name, risk);
            }
        }
        return map;
    }

    @Override
    public ResponseData getDis(SearchData searchData) {
        ResponseData responseData = new ResponseData();
        String[] featureTypes = searchData.getFeatureType().split(",");
        List<String> featureTypeList = Arrays.asList(featureTypes);
        logger.info("前端传来的features :"+featureTypeList);
        List<String> newList = new ArrayList<>();
        List<String> inputList = new ArrayList<>();
        Map<String, Map<String, String>> inputs = searchData.getInputs();
        Integer diseaseType = searchData.getDisType();
        logger.info("diseaseType为 :"+diseaseType);
        Set<String> ss = inputs.keySet();
        logger.info("分词系统传来的词: "+ss);
        logger.info("图谱开始根据传来的分词计算诊断.....");
        inputList.addAll(ss);
        //图谱计算是否有组合词
        List<Map<String, Object>> collectionWord = baseNodeRepository.getCollectionWord(inputList);
        if(collectionWord !=null && collectionWord.size()>0){
            for (Map<String, Object> word:collectionWord) {
                String jundgement = word.get("jundgement").toString();
                String standName = word.get("standName").toString();
                if("TRUE".equals(jundgement)){
                    inputList.add(standName);
                }
            }
        }
        List<Map<String, Object>> collectJYWord = baseNodeRepository.getCollectJYWord(inputList);
        if(collectJYWord !=null && collectJYWord.size()>0){
            for (Map<String, Object> word:collectJYWord) {
                String fild = word.get("fild").toString();
                String[] typeCollect = (String[])word.get("typeCollect");
                if(typeCollect.length == 1 && "近义词".equals(typeCollect[0])){
                    List<Map<String, Object>> jyWord = baseNodeRepository.getJYWord(fild);
                    if(jyWord !=null && jyWord.size()>0){
                        for (Map<String, Object> ji:jyWord) {
                            String js = ji.get("js").toString();
                            inputList.remove(fild);
                            inputList.add(js);
                        }
                    }
                }
            }
        }
        newList.addAll(inputList);
        while (newList.size()>0){
            List<Map<String, Object>> condition = baseNodeRepository.getCondition(newList, inputList);
            newList.clear();
            if(condition !=null && condition.size()>0){
                for (Map<String, Object> cd:condition) {
                    String conditionName = cd.get("condition").toString();
                    String jundgement = cd.get("jundgement").toString();
                    if ("true".equals(jundgement)) {
                        newList.add(conditionName);
                        inputList.add(conditionName);
                    }
                }
            }
        }
        List<String> que = new ArrayList<>();
        List<Map<String, Object>> disease = baseNodeRepository.getDisease(inputList);
        if(disease !=null && disease.size()>0){
            for (Map<String, Object> d: disease) {
                String diseaseName = d.get("name").toString();
                String relationType = d.get("relationType").toString();
                if("确诊".equals(relationType) || "拟诊".equals(relationType)){
                    que.add(diseaseName);
                }
            }
        }
        logger.info("图谱计算出来的诊断为: "+que);
        List<FeatureRate> featureRates = new ArrayList<>();
        if(que !=null && que.size()>0){
            for (String quezhen:que) {
                if(!"低血糖反应".equals(quezhen) && !"胃肠道不良反应".equals(quezhen)
                        && !"肾功能不全".equals(quezhen)){
                    FeatureRate featureRate = new FeatureRate();
                    featureRate.setFeatureName(quezhen);
                    featureRate.setExtraProperty("");
                    featureRate.setDesc(quezhen);
                    featureRate.setRate("neo4j");
                    featureRates.add(featureRate);
                }
            }
        }
        String webDiag = searchData.getDiag();
        logger.info("界面输入的诊断为: "+webDiag);
        if(webDiag.trim()!=null && webDiag.trim() !="" && featureTypeList.contains("8")){
            logger.info("界面有诊断,并且features包含8,可以走治疗方案");
            Map<String, Filnlly> mulDiseaseTreat = processTreat(webDiag, ss,diseaseType,que);
            responseData.setTreat(mulDiseaseTreat);
        }
        if(featureTypeList.contains("22") && que != null && que.size()>0){
            logger.info("图谱推送有疑诊,并且features包含22,可以走右侧推送");
            List<MedicalIndication> medicalIndicationList = new ArrayList<>();
            MedicalIndication medicalIndication = this.getMedicalIndication(que);
            medicalIndicationList.add(medicalIndication);
            responseData.setMedicalIndications(medicalIndicationList);
        }
        if(featureTypeList.contains("11") && diseaseType == 1 && diseaseType !=null) {
            logger.info("featureTypeList 包含11,走管理评估!!!");
//            MangementEvaluation mangementEvaluation = this.getMangementEvaluation();
//            Map<String, JSONObject> mangementEvaluation1 = mangementEvaluation.getMangementEvaluation();
//            responseData.setManagementEvaluation(mangementEvaluation1);
            Map test = this.getTest();
            responseData.setManagementEvaluation(test);
        }
        responseData.setDis(featureRates);
        return responseData;
    }

    @Override
    public Map<String, List<FeatureRate>> getLisPacs(SearchData searchData) {
        Map<String, List<FeatureRate>> lisPacsFeature = new HashMap<>();
        String webDiag = searchData.getDiag();
        List<String> webDiagList = null;
        if(webDiag != null && webDiag !="") {
            webDiagList = Arrays.asList(webDiag.split(",|，|、"));
        }
            List<FeatureRate> bigdataDiagFeature = searchData.getPushDiags();
            List<String> bigdataDiagList =new LinkedList<>();
            if(bigdataDiagFeature.size()>0){
                for (FeatureRate fe:bigdataDiagFeature) {
                    if(!"{\"鉴别诊断\":\"\"}".equals(fe.getDesc())){
                        if("neo4j".equals(fe.getRate())){
                            bigdataDiagList.add(fe.getFeatureName());
                            logger.info("图谱推出的诊断为: "+fe.getFeatureName());
                        }else {
                            bigdataDiagList.add(fe.getFeatureName());
                            logger.info("大数据推出的诊断为: "+fe.getFeatureName());
                        }
                    }
                }
            }
            if(webDiagList != null){
                for (String web: webDiagList ) {
                    for (int i=0;i<bigdataDiagList.size();i++ ) {
                        if(bigdataDiagList.get(i).equals(web)){
                            bigdataDiagList.remove(bigdataDiagList.get(i));
                        }
                    }
                }
            }
            logger.info("界面诊断为: " + webDiagList);
            Map<String, Set<String>> weblisPacs =null;
            Map<String, Set<String>> biglisPacs =null;
            if(webDiagList !=null && webDiagList.size()>0){
                weblisPacs = processLisPacs(webDiagList);
            }
            if(bigdataDiagList !=null && bigdataDiagList.size()>0){
                biglisPacs = processLisPacs(bigdataDiagList);
            }
            Set<String> lis =null;
            Set<String> pacs =null;
            //如果界面有诊断
            if(weblisPacs !=null && weblisPacs.values().size()>0){
                lis = weblisPacs.get("LIS");
                pacs = weblisPacs.get("PACS");
                logger.info("界面有诊断的情况下,界面诊断推出的lis为: "+lis);
                logger.info("界面有诊断的情况下,界面诊断推出的pacs为: "+pacs);
                if(biglisPacs !=null && biglisPacs.values().size()>0){
                    Set<String> bl = biglisPacs.get("LIS");
                    Set<String>  bp= biglisPacs.get("PACS");
                    logger.info("界面有诊断的情况下,推出诊断的lis为: "+bl);
                    logger.info("界面有诊断的情况下,推出诊断的pacs为: "+bp);
                    lis.addAll(bl);
                    pacs.addAll(bp);
                }
            }else if(biglisPacs !=null ) {
                lis = biglisPacs.get("LIS");
                pacs = biglisPacs.get("PACS");
                logger.info("界面无诊断的情况下,推出诊断的lis为: "+lis);
                logger.info("界面无诊断的情况下,推出诊断的lis为: "+pacs);
            }
            logger.info("推出的合并lis为: " + lis);
            logger.info("推出的合并pacs为: " + pacs);
            ArrayList<FeatureRate> lisFeature = new ArrayList<>();
            ArrayList<FeatureRate> pacsFeature = new ArrayList<>();
            if(lis !=null && lis.size()>0){
                for (String l:lis) {
                    FeatureRate featureRate = new FeatureRate();
                    featureRate.setFeatureName(l);
                    lisFeature.add(featureRate);
                }
            }
           if(pacs !=null && pacs.size()>0){
               for (String p:pacs) {
                   FeatureRate featureRate = new FeatureRate();
                   featureRate.setFeatureName(p);
                   pacsFeature.add(featureRate);
               }
           }
            lisPacsFeature.put("lisList",lisFeature);
            lisPacsFeature.put("pacsList",pacsFeature);

        return lisPacsFeature;
    }

    public Map<String, Set<String>> processLisPacs(List<String> webDiagList) {
        Map<String, Set<String>> lisPacsMap = new HashMap<>();
        List<String> lisArray = new ArrayList<>();
        List<String> pacsArray = new ArrayList<>();
        List<Map<String, Object>> lisPacsList = baseNodeRepository.getLisPacsList(webDiagList);
        if(lisPacsList !=null && lisPacsList.size()>0){
            for (Map<String, Object> f:lisPacsList) {
                String label = f.get("label").toString();
                String name = f.get("name").toString();
                if("LIS".equals(label)){
                    lisArray.add(name);
                }else if("PACS".equals(label)){
                    pacsArray.add(name);
                }
            }
        }
        lisPacsMap.put("LIS",processList(lisArray));
        lisPacsMap.put("PACS",processList(pacsArray));
        return lisPacsMap;
    }

    /**
     * 处理治疗
     * @param
     */
    public Map<String, Filnlly> processTreat(String webDisease,Set<String> inputFilds,Integer disType,List<String> que){
        Map<String, Filnlly> diagTreat = new HashMap<>();
        NumberFormat nf = NumberFormat.getPercentInstance();
        nf.setMinimumFractionDigits(0);//设置该百分比数字，保留2位小数;
        nf.setRoundingMode(RoundingMode.HALF_UP); //设置满5向上进位，即四舍五入;
        String[] diseaseArray = webDisease.split(",|，|、|;|:|；");
        List<String> diseaseList = Arrays.asList(diseaseArray);
        Map<String, List> diseFilds = new HashMap<>();
        for(int i=0;i<diseaseList.size();i++){
            List<String> fildsList = new ArrayList<>();//其他条件数组
            for (String fild:inputFilds) {
                fildsList.add(fild);
            }
            for (int j = 0; j < diseaseList.size(); j++) {
                if (i != j) {
                    fildsList.add(diseaseList.get(j));
                }
            }
            diseFilds.put(diseaseList.get(i), fildsList);
        }
        for (Map.Entry<String, List> disFild:diseFilds.entrySet()) {
            //获取诊断的药类和用药(排序好的)
            List<Map<String, Object>> treatMedicaton = baseNodeRepository.getTreatMedicaton(disFild.getKey());
            Map<String, LinkedHashMap<String, String>> contentMap = new LinkedHashMap<>();
            if(treatMedicaton != null && treatMedicaton.size()>0){
                for (Map<String, Object> tm:treatMedicaton) {
                    String drugs = tm.get("drugs").toString();
                    String medication = tm.get("medication").toString();
                    String rate = tm.get("rate").toString();
                    if (contentMap.containsKey(drugs)) {
                        LinkedHashMap<String, String> drugsMedicationLinkedHashMap = contentMap.get(drugs);
                        drugsMedicationLinkedHashMap.put(medication,rate);
                        contentMap.put(drugs,drugsMedicationLinkedHashMap);
                    }else {
                        LinkedHashMap<String, String> sa = new LinkedHashMap<>();
                        sa.put(medication,rate);
                        contentMap.put(drugs,sa);
                    }
                }
            }
            //药所属类型的大小类合并
            Map<String, String> shortLargeMap = new HashMap<>();
            List<Map<String, Object>> shortLarge = baseNodeRepository.getShortLarge(disFild.getKey());
            if(shortLarge != null && shortLarge.size()>0){
                for (Map<String, Object> sl:shortLarge) {
                    String aShort = sl.get("short").toString();
                    String large = sl.get("large").toString();
                    shortLargeMap.put(aShort,large+"("+aShort+")");
                }
            }
            //小类替换成大类
            Map<String, LinkedHashMap<String, String>> shortLargeMapConcat = new LinkedHashMap<>();
            for (Map.Entry<String, LinkedHashMap<String, String>> f : contentMap.entrySet()) {
                String drugs = f.getKey();
                if (shortLargeMap.get(drugs) != null) {
                    shortLargeMapConcat.put(shortLargeMap.get(drugs), contentMap.get(drugs));
                } else {
                    shortLargeMapConcat.put(drugs, contentMap.get(drugs));
                }
            }
            //处理慎用,忌用
            Map<String, String> medicationUseMap = new HashMap<>();//key:药名 value:禁忌(慎用)
            List<Map<String, Object>> shenJi = baseNodeRepository.getShenJiDrugsMedication(disFild.getKey(), disFild.getValue());
            if(shenJi != null  && shenJi.size()>0){
                for (Map<String, Object> sj:shenJi) {
                    String name = sj.get("name").toString();
                    String type = sj.get("type").toString();
                    medicationUseMap.put(name,type);//(名字,禁用/慎用)
                }
            }
            //大对象集合
            ArrayList<Drugs> drugsList = new ArrayList<Drugs>();
            DrugsPattern drugsPattern = new DrugsPattern();
            for (Map.Entry<String, LinkedHashMap<String, String>> w : shortLargeMapConcat.entrySet()) {
                int i = 0;
                int forbidden = 0;
                Drugs drugs = new Drugs();
                String drugsName = w.getKey();
                Map<String, String> bigSubDrugs = drugsPattern.bigSubDrugs(drugsName);
                drugs.setBigdrugsName(bigSubDrugs.get("big"));
                drugs.setSubdrugsName(bigSubDrugs.get("sub"));
                LinkedList<Medicition> medicitionsList = new LinkedList<>();
                LinkedHashMap<String, String> meditionRate = w.getValue();
                if ("忌用".equals(medicationUseMap.get(bigSubDrugs.get("big"))) || "忌用".equals(medicationUseMap.get(bigSubDrugs.get("sub")))) {
                    drugs.setDrugsForbidden(2);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        medicition.setForbidden(2);
                        medicitionsList.add(medicition);
                    }
                }else if("慎用".equals(medicationUseMap.get(bigSubDrugs.get("big"))) || "慎用".equals(medicationUseMap.get(bigSubDrugs.get("sub")))){
                    drugs.setDrugsForbidden(1);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        medicition.setForbidden(1);
                        medicitionsList.add(medicition);
                    }
                }else {
                    drugs.setDrugsForbidden(0);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        if ("忌用".equals(medicationUseMap.get(meditionName))) {
                            medicition.setForbidden(2);
                        } else if ("慎用".equals(medicationUseMap.get(meditionName))) {
                            medicition.setForbidden(1);
                        } else {
                            medicition.setForbidden(0);
                        }
                        medicitionsList.add(medicition);
                    }
                }
                drugs.setMedicitionsList(medicitionsList);
                drugsList.add(drugs);
            }
            Filnlly filnlly = new Filnlly();
            //走不良反应
            if(disType !=null && 1==disType){
                logger.info("disType=1,可以走不良反应!!");
                List<Indicators> indicatorsList1 = new ArrayList<>();
                Indicators indicators1 = getAdverse(que, "低血糖反应");
                Indicators indicators2 = getAdverse(que, "胃肠道不良反应");
                indicatorsList1.add(indicators1);
                indicatorsList1.add(indicators2);
                filnlly.setAdverseEvent(indicatorsList1);
            }
            filnlly.setTreatment(drugsList);
            diagTreat.put(disFild.getKey(),filnlly);
        }
        return diagTreat;
    }

    /**
     *  量表和指标
     * @param diseaseSet 图谱推出的诊断
     * @return
     */
    public MedicalIndication getMedicalIndication(List<String> diseaseSet){
        MedicalIndication medicalIndication = new MedicalIndication();
        if(diseaseSet.contains("肾功能不全")){
            logger.info("diseaseSet中包含肾功能不全");
            //推送指标
            medicalIndication.setName("肾功能不全");
            List<MedicalIndicationDetail> medicalIndicationDetailList = new ArrayList<>();
            //量表
            MedicalIndicationDetail medicalIndicationDetail1 = new MedicalIndicationDetail();
            medicalIndicationDetail1.setType(1);
            JSONObject liangJson = new JSONObject();
            liangJson.put("name","密西根糖尿病周围神经病评分（MDNS）");
            medicalIndicationDetail1.setContent(liangJson);
            //公式
            MedicalIndicationDetail medicalIndicationDetail2 = new MedicalIndicationDetail();
            medicalIndicationDetail2.setType(2);
            JSONObject gongshiJson = new JSONObject();
            gongshiJson.put("name","肾小球滤过率");
            List<JSONObject> detailList = new ArrayList<>();
            JSONObject ageJsonObject = new JSONObject();
            ageJsonObject.put("controlType",2);
            ageJsonObject.put("name","年龄");
            ageJsonObject.put("value","");
            ageJsonObject.put("uint","");
            ageJsonObject.put("details",new ArrayList<>());
            JSONObject indicationJsonObject = new JSONObject();
            indicationJsonObject.put("controlType",2);
            indicationJsonObject.put("name","血肌酐");
            indicationJsonObject.put("value","");
            indicationJsonObject.put("uint","mg/dL");
            indicationJsonObject.put("details",new ArrayList<>());
            JSONObject sexJson = new JSONObject();
            sexJson.put("controlType",1);
            sexJson.put("name","性别");
            sexJson.put("value","");
            sexJson.put("uint","");
            List<JSONObject> sexList = new ArrayList<>();
            JSONObject manJson = new JSONObject();
            manJson.put("detailName","男");
            manJson.put("value",1);
            manJson.put("state",0);
            JSONObject womanJson = new JSONObject();
            womanJson.put("detailName","女");
            womanJson.put("value",1.018);
            womanJson.put("state",1);
            sexList.add(manJson);
            sexList.add(womanJson);
            sexJson.put("details",sexList);
            detailList.add(ageJsonObject);
            detailList.add(indicationJsonObject);
            detailList.add(sexJson);
            gongshiJson.put("details",detailList);
            JSONObject resultJson = new JSONObject();
            resultJson.put("value","");
            resultJson.put("unit","");
            resultJson.put("text","");
            gongshiJson.put("result",resultJson);
            medicalIndicationDetail2.setContent(gongshiJson);
            //其他
            MedicalIndicationDetail medicalIndicationDetail3 = new MedicalIndicationDetail();
            medicalIndicationDetail3.setType(3);
            JSONObject shiFouJson = new JSONObject();
            shiFouJson.put("name","是否");
            shiFouJson.put("controlType",0);
            List<JSONObject> shiFouList = new ArrayList<>();
            JSONObject shiJsonObject = new JSONObject();
            shiJsonObject.put("detailName","是");
            shiJsonObject.put("state",0);
            JSONObject fouJsonObject = new JSONObject();
            fouJsonObject.put("detailName","否");
            fouJsonObject.put("state",1);
            shiFouList.add(shiJsonObject);
            shiFouList.add(fouJsonObject);
            shiFouJson.put("details",shiFouList);
            medicalIndicationDetail3.setContent(shiFouJson);
            medicalIndicationDetailList.add(medicalIndicationDetail1);
            medicalIndicationDetailList.add(medicalIndicationDetail2);
            medicalIndicationDetailList.add(medicalIndicationDetail3);
            medicalIndication.setDetails(medicalIndicationDetailList);
        }
        return medicalIndication;
    }

    /**
     * 管理评估
     * @return
     */
    public MangementEvaluation getMangementEvaluation(){
        MangementEvaluation mangementEvaluation = new MangementEvaluation();
        Map<String,JSONObject> mangementMap = new HashMap<>();
        JSONObject curativeJson = new JSONObject();
        List<JSONObject> itemJson = new ArrayList<>();
        JSONObject bloodPressure = new JSONObject();
        bloodPressure.put("controltype",0);
        bloodPressure.put("name","血压控制情况");
        List<JSONObject> bloodPressureLevelList = new ArrayList<>();
        String[] items= {"正常","1级","2级","3级"};
        for (String item:items) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bloodPressureLevelList.add(iJson);
        }
        bloodPressure.put("details",bloodPressureLevelList);
        itemJson.add(bloodPressure);

        JSONObject bmiControl = new JSONObject();
        bmiControl.put("controltype",0);
        bmiControl.put("name","BMI控制情况");
        List<JSONObject> bmiControlLevelList = new ArrayList<>();
        String[] bmiControlitems= {"体重过低","体重达标","肥胖前期","肥胖"};
        for (String item:bmiControlitems) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bmiControlLevelList.add(iJson);
        }
        bmiControl.put("details",bmiControlLevelList);
        itemJson.add(bmiControl);

        JSONObject bloodFatControl = new JSONObject();
        bloodFatControl.put("controltype",0);
        bloodFatControl.put("name","血脂控制情况");
        List<JSONObject> bloodFatControlLevelList = new ArrayList<>();
        String[] bloodFatControlitems= {"高血脂"};
        for (String item:bloodFatControlitems) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bloodFatControlLevelList.add(iJson);
        }
        bloodFatControl.put("details",bmiControlLevelList);
        itemJson.add(bloodFatControl);
        curativeJson.put("rows",itemJson);
        //分级管理
        JSONObject gradeJson = new JSONObject();
        List<JSONObject> gradItemJson = new ArrayList<>();
        JSONObject bigGrad = new JSONObject();
        bigGrad.put("name","血压控制情况");
        bigGrad.put("controltype",0);
        List<JSONObject> gradItemJ = new ArrayList<>();
        String[] gradItem ={"常规管理","强化管理"};
        for (String item:gradItem) {
            JSONObject j =new JSONObject();
            j.put("detailName",item);
            j.put("state",0);
            gradItemJ.add(j);
        }
        bigGrad.put("details",gradItemJ);
        gradItemJson.add(bigGrad);
        gradeJson.put("rows",gradItemJson);
        mangementMap.put("疗效评估",curativeJson);
        mangementMap.put("糖尿病分级管理",gradeJson);
        mangementEvaluation.setMangementEvaluation(mangementMap);
        return mangementEvaluation;
    }

    public Map getTest(){
        Map result = new HashMap();
        //疗效评估
        Map liaoXiaoMap = new HashMap();
        List<Map> rowList = new ArrayList<>();
        Map item1 = new HashMap();
        item1.put("controltype",0);
        item1.put("name","血压控制情况");
        List<Map> item1DetailList = new ArrayList<>();
        Map de1= new HashMap();
        de1.put("detailName","正常");
        de1.put("state",1);
        Map de2= new HashMap();
        de2.put("detailName","1级");
        de2.put("state",0);
        Map de3= new HashMap();
        de3.put("detailName","2级");
        de3.put("state",0);
        Map de4= new HashMap();
        de4.put("detailName","3级");
        de4.put("state",0);
        item1DetailList.add(de1);
        item1DetailList.add(de2);
        item1DetailList.add(de3);
        item1DetailList.add(de4);
        item1.put("details",item1DetailList);

        Map item2 = new HashMap();
        item2.put("controltype",1);
        item2.put("name","BMI控制情况");
        List<Map> item2DetailList = new ArrayList<>();
        Map be1= new HashMap();
        be1.put("detailName","体重过低");
        be1.put("state",1);
        Map be2= new HashMap();
        be2.put("detailName","体重达标");
        be2.put("state",0);
        Map be3= new HashMap();
        be3.put("detailName","肥胖前期");
        be3.put("state",0);
        Map be4= new HashMap();
        be4.put("detailName","肥胖");
        be4.put("state",0);
        item2DetailList.add(be1);
        item2DetailList.add(be2);
        item2DetailList.add(be3);
        item2DetailList.add(be4);
        item2.put("details",item2DetailList);

        Map item3 = new HashMap();
        item3.put("controltype",0);
        item3.put("name","血脂控制情况");
        List<Map> item3DetailList = new ArrayList<>();
        Map xe1= new HashMap();
        xe1.put("detailName","高血脂");
        xe1.put("state",1);
        item3DetailList.add(xe1);
        item3.put("details",item3DetailList);

        rowList.add(item1);
        rowList.add(item2);
        rowList.add(item3);
        liaoXiaoMap.put("rows",rowList);
        //分级管理
        Map fenjiMap = new HashMap();
        List<Map> ferowList = new ArrayList<>();
        Map feitem1 = new HashMap();
        feitem1.put("controltype",0);
        feitem1.put("name","血压控制情况");
        List<Map> feDetailList = new ArrayList<>();
        Map fe1= new HashMap();
        fe1.put("detailName","常规管理");
        fe1.put("state",1);
        Map fe2= new HashMap();
        fe2.put("detailName","强化管理");
        fe2.put("state",0);
        feDetailList.add(fe1);
        feDetailList.add(fe2);
        feitem1.put("details",feDetailList);
        ferowList.add(feitem1);
        fenjiMap.put("rows",ferowList);

        result.put("疗效评估",liaoXiaoMap);
        result.put("糖尿病分级管理",fenjiMap);

        return result;
    }

    /**
     * 不良反应
     * @param dis
     * @param name
     * @return
     */
    public Indicators getAdverse(List<String> dis,String name){
        List<Detail> detailList1 = new ArrayList<>();
        Indicators indicators1 = new Indicators();
        indicators1.setName(name);
        indicators1.setControlType(2);
        Detail detail1 = new Detail();
        detail1.setName("否");
        detail1.setValue(0);
        Detail detail2 = new Detail();
        detail2.setName("是");
        detail2.setValue(0);
        if(dis.contains(name)){
            detail2.setValue(1);
        }
        Detail detail3 = new Detail();
        detail3.setName("轻度");
        detail3.setValue(0);
        Detail detail4 = new Detail();
        detail4.setName("中度");
        detail4.setValue(0);
        Detail detail5 = new Detail();
        detail5.setName("重度");
        detail5.setValue(0);
        Detail detail6 = new Detail();
        detail6.setName("偶尔");
        detail6.setValue(0);
        Detail detail7 = new Detail();
        detail7.setName("频繁");
        detail7.setValue(0);
        detailList1.add(detail1);
        detailList1.add(detail2);
        detailList1.add(detail3);
        detailList1.add(detail4);
        detailList1.add(detail5);
        detailList1.add(detail6);
        detailList1.add(detail7);
        indicators1.setDetails(detailList1);
        return indicators1;
    }
    @Override
    public List<Map<String, Object>> getRecommendItem(QueryVo queryVo) {
        List<Map<String, Object>> list = relationRepository.getRecommendItem(queryVo.getDiseaselist());
        return list;
    }

    @Override
    public Map<String, List<Map<String, Object>>> getScale(QueryVo queryVo) {
        Map<String, List<Map<String, Object>>> map = new HashMap<>();

        for (String name:queryVo.getScalelist()) {
            List<Map<String, Object>> list = scaleRepository.getScale(name);
            map.put(name, list);
        }

        return map;
    }


    public Pageable getPageable(PageVo pageVo) {
        if (pageVo.getSize() > 0) {
            if (pageVo.getSort_key() != null && pageVo.getSort_direct() != null) {
                if (pageVo.getSort_direct().toLowerCase().equals("asc")) {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize(),
                            Sort.Direction.ASC, pageVo.getSort_key().trim());
                }
                else if (pageVo.getSort_direct().toLowerCase().equals("desc")) {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize(),
                            Sort.Direction.DESC, pageVo.getSort_key().trim());
                }
                else {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize());
                }
            }
            else {
                pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize());
            }
        }
        else {
            pageable = PageRequest.of(0, 100);
        }
        return pageable;
    }

    public List<Map<String, Object>> extractPath(Iterable<Map<String, Object>> path) {
        Map<String, Object> item = new HashMap<>();
        List<Map<String, Object>> pathlist = new ArrayList<>();
        Map<String, Object> pathmap;

        Iterator<Map<String, Object>> iterator = path.iterator();
        while (iterator.hasNext()) {
            item = iterator.next();
            pathmap = new HashMap<>();
            if (item.get("P") != null) {
                Path p = (Path)item.get("P");
                pathmap = parsePath(p);
            }
            pathlist.add(pathmap);
        }

        return pathlist;
    }

    public Map<String, Object> parsePath(Path p) {
        Map<String, Object> path = new HashMap<>();
        path.put("start", new HashMap<>());
        path.put("end", new HashMap<>());
        Map<String, String> s = (Map)path.get("start");
        Map<String, String> e = (Map)path.get("end");
        Node start = p.start();
        Node end = p.end();
        s.put("label", start.labels().iterator().next());
        e.put("label", end.labels().iterator().next());
        String relation = p.relationships().iterator().next().type();
        path.put("relation", new HashMap<>());
        ((Map)path.get("relation")).put("type", relation);

        for (String prop:start.keys()) {
            s.put(prop, start.get(prop).toString().replaceAll("\"",""));
        }

        for (String prop:end.keys()) {
            e.put(prop, end.get(prop).toString().replaceAll("\"",""));
        }

        return path;
    }

    /**
     * 根据出现次数排序,去重
     * @param list
     * @return
     */
    public Set<String> processList(List<String> list) {
        Set<String> resultSet = new LinkedHashSet<>();
        Map<String, Integer> featuerInt = new HashMap<String, Integer>();
        for (String d : list) {
            if (featuerInt.containsKey(d)) {
                featuerInt.put(d, featuerInt.get(d) + 1);
            } else {
                featuerInt.put(d, 1);
            }
        }
        List<Map.Entry<String, Integer>> reslist = new ArrayList<Map.Entry<String, Integer>>(featuerInt.entrySet());
        Collections.sort(reslist, new Comparator<Map.Entry<String, Integer>>() {
            @Override
            public int compare(Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2) {
                return o2.getValue().compareTo(o1.getValue());
            }
        });
        for (Map.Entry<String, Integer> f : reslist
                ) {
            //排好序的set
            resultSet.add(f.getKey());
        }
        return resultSet;
    }

}
